/*
  Board: VT1100 Mini
  Example: Simple Sleep
  Description: A simple example that starts the CC2530 as a End Device, sets all poll rates to zero and sleeps using the watchdog timer.
  The device performs the following Loop:
  1. Send data
  2. Sleep
*/

/* ------------------------------------------------------------------
   Includes
   ------------------------------------------------------------------
*/
#include <VT1100_Mini.h>
#include <SPI.h>

/* ------------------------------------------------------------------
   Pins
   ------------------------------------------------------------------
*/
#define PIN_EN      7                                                 // Enable
#define PIN_SRDY    8                                                 // (P0_4) SRDY: Slave ready.
#define PIN_RES     9                                                 // (RST) RESET
#define PIN_SS_MRDY 10                                                // (P0_3) (P1_4) SS: Slave select.  MRDY: Master Ready.  Active Low.  Connect SS and MRDY together
#define PIN_MOSI    11                                                // (P1_6) SPI MO: Master-output slave-input data
#define PIN_MISO    12                                                // (P1_7) SPI MI: Master-input slave-output data
#define PIN_SCK     13                                                // (P1_5) SPI Serial Clock

#define InitButton 2                                                  // Button on Digital Pin 2 (D2)

/* ------------------------------------------------------------------
   Library Class Instances
   ------------------------------------------------------------------
*/
CC2530 myCC2530;                                                      // Make an instance of the class from the Library.  A short name for referring to variables and functions from the VT1100_Mini Library

/* ------------------------------------------------------------------
   Global Variables
   ------------------------------------------------------------------
*/
uint8_t PollFailureCounter = 0;

/********************************************************************
   Setup
 ********************************************************************
*/
void setup()
{
  pinMode(InitButton, INPUT_PULLUP);                                  // Button to control if the CC2530 should be commissioned.

  SPI.begin();
  Serial.begin(115200);
  delay(100);

  // Parameters for CC2530
  myCC2530.SetPANID(0x00A2);                                          // PAN ID.  Two bytes set between 0x0000 and 0x3FFF.  Examples: 0x00A1, 0x00A2 or 0x00A3.
  myCC2530.SetLOGICAL_TYPE(0x02);                                     // Device type.  Examples: Coordinator = 0x00, Roouter = 0x01 or End Device = 0x02
  myCC2530.SetCHANLIST(11);                                           // Wireless Channel. Examples: 11 to 26
  myCC2530.SetPOLL_FAILURE_RETRIES(255);                              // Set poll failure retries to 255 to ensure a rejoin isn't started on loss of parent device.  To reduce battery drain.

  Init_CC2530();                                                      // Power up and commission the CC2530
  myCC2530.WriteNV_POLLRATE(0);                                       // Writes to non-volitile memory to set Poll Rate. Must be run after joining process.  The time period in milliseconds to poll the parent for messages.
  myCC2530.WriteNV_QUEUED_POLL_RATE(0);                               // The time period to poll again for more data if the parent does have data.
  myCC2530.WriteNV_RESPONSE_POLL_RATE(0);                             // The time period to poll after sending a AF_Data_Request to poll for acknowledgement of recieved messages.
  myCC2530.ZDO_SET_REJOIN_PARAMETERS(900000, 30000);                  // Reduce rejoin scan duration of orphan scan when parent is lost
}

/********************************************************************
   Loop
 ********************************************************************
*/
void loop()
{
  /* ------------------------------------------------------------------
    Send Data
    -------------------------------------------------------------------
  */
  uint8_t Data = 0x08;                                                // Data to send
  myCC2530.AF_DATA_REQUEST(0x00, 0x00, Data, sizeof(Data));           // AF_DATA_REQUEST(Short Address, Short Address, Variable to send, Length).  The coordinators address is always 0x0000.
  if (myCC2530.AF_DATA_CONFIRM(150))                                  // Function returns true if message acknowledgement (ACK) is success and false if it fails. The maximum amout of time to wait for message acknowledgement (ACK).  See "Z-stack Duty Cycle Analysis" Application Note AN114.  Within 100 ms, as many as 3 application retries can be attempted.
  {
    Serial.println("ACK Confirm");
    PollFailureCounter = 0;
  }
  else
  {
    Serial.println("ACK Fail");
    PollFailureCounter++;
  }

  SleepForeverPollFailures(100);                                      // If consecutive poll failures are larger than a set variable, sleep forever to conserve energy.  This function stops the CC2530 ever going into Orphan state.

  /* -----------------------------------------------------------------
    Sleep
    ------------------------------------------------------------------
  */
  SetPinsSleep();
  myCC2530.SLEEP_WATCHDOG_TIMER(8);                                   // Sleep for approximately 15min (8sec*113).  After 255 failed retries the CC2530 will try and rejoin which causes battery drain.  Increase the time before this mechanism is activated to ensure battery conservation.
  SetPinsWake();
}

/********************************************************************
   Other Functions
 ********************************************************************
*/

/* ------------------------------------------------------------------
   Initialise CC2530
   ------------------------------------------------------------------
*/
void Init_CC2530()
{
  myCC2530.POWER_UP();                                                // Power up the CC2530 (wake on reset).

  if (digitalRead(InitButton) == LOW)
  {
    myCC2530.COMMISSION();                                            // Clears the configuration and network state then writes the new configuration parameters to the CC2530 non-volitile (NV) memory.  This should only be run once on initial setup.
  }

  myCC2530.AF_REGISTER(0x01);                                         // Register Endpoint
  myCC2530.ZDO_STARTUP_FROM_APP(60000);                               // Starts the CC2530 in the network
  delay(2000);
}

/* ------------------------------------------------------------------
   Sleep Forever Poll Failures
   ------------------------------------------------------------------
*/
void SleepForeverPollFailures(uint8_t consecutivePollFailures)
{
  if (PollFailureCounter >= consecutivePollFailures)
  {
    Serial.print("Consecutive Poll failures >= ");
    Serial.println(PollFailureCounter);
    Serial.println("Sleep Forever");
    delay(50);
    myCC2530.SLEEP_EXT_INTERRUPT();
  }
}

/* ------------------------------------------------------------------
   SetPinsSleep
   Description: Set pin states Output LOW before Sleep
   ------------------------------------------------------------------
*/
void SetPinsSleep()
{
  for (byte i = 2; i <= 6; i++)                                           // Set pins output low
  {
    pinMode (i, OUTPUT);
    digitalWrite (i, LOW);
  }

  for (byte i = A0; i <= A5; i++)
  {
    pinMode (i, OUTPUT);
    digitalWrite (i, LOW);
  }
}

/* ------------------------------------------------------------------
   SetPinsWake
   Description: Set pins Default inputs after sleep
   ------------------------------------------------------------------
*/
void SetPinsWake()
{
  for (byte i = 2; i <= 6; i++)                                       // Set pins default input
  {
    pinMode (i, INPUT);
  }

  for (byte i = A0; i <= A5; i++)
  {
    pinMode (i, INPUT);
  }
}
