/*
  Board: VT1100 Mini
  Example: Sleep External Interrupt
  Description: A simple example that starts the CC2530 as a End Device, polls for data, sends data and sleeps.  Sleep mode can be interrupted by an external interrupt on pin D2 or when the device polls for data.
               If the CC2530 polls for data and a message is available the SRDY pin will be set low and trigger a PC interrupt for the application processor (Atmega328) to receive the message.
  The device performs the following Loop:
  1. Send data
  2. Sleep
  3. Wake on External or PC Interrupt
*/

/* ------------------------------------------------------------------
   Includes
   ------------------------------------------------------------------
*/
#include <VT1100_Mini.h>
#include <SPI.h>

/* ------------------------------------------------------------------
   Pins
   ------------------------------------------------------------------
*/
#define PIN_EN      7                                                 // Enable
#define PIN_SRDY    8                                                 // (P0_4) SRDY: Slave ready.
#define PIN_RES     9                                                 // (RST) RESET
#define PIN_SS_MRDY 10                                                // (P0_3) (P1_4) SS: Slave select.  MRDY: Master Ready.  Active Low.  Connect SS and MRDY together
#define PIN_MOSI    11                                                // (P1_6) SPI MO: Master-output slave-input data
#define PIN_MISO    12                                                // (P1_7) SPI MI: Master-input slave-output data
#define PIN_SCK     13                                                // (P1_5) SPI Serial Clock

#define InitButton 2                                                  // Button on Digital Pin 2 (D2)

/* ------------------------------------------------------------------
   Library Class Instances
   ------------------------------------------------------------------
*/
CC2530 myCC2530;                                                      // Make an instance of the class from the Library.  A short name for referring to variables and functions from the VT1100_Mini Library

/********************************************************************
   Setup
 ********************************************************************
*/
void setup()
{
  pinMode(InitButton, INPUT_PULLUP);                                  // Button to control if the CC2530 should be commissioned.

  SPI.begin();
  Serial.begin(115200);
  delay(100);

  // Parameters for CC2530
  myCC2530.SetPANID(0x00A2);                                          // PAN ID.  Two bytes set between 0x0000 and 0x3FFF.  Examples: 0x00A1, 0x00A2 or 0x00A3.
  myCC2530.SetLOGICAL_TYPE(0x02);                                     // Device type.  Examples: Coordinator = 0x00, Roouter = 0x01 or End Device = 0x02
  myCC2530.SetCHANLIST(11);                                           // Wireless Channel. Examples: 11 to 26
  myCC2530.SetPOLL_FAILURE_RETRIES(3);                                // Set poll failure retries

  Init_CC2530();                                                      // Power up and commission the CC2530
  myCC2530.WriteNV_POLLRATE(2000);                                    // Writes to non-volitile memory to set Poll Rate. Must be run after joining process.  The time period in milliseconds to poll the parent for messages.
  myCC2530.WriteNV_QUEUED_POLL_RATE(100);                             // The time period to poll again for more data if the parent does have data.
  myCC2530.WriteNV_RESPONSE_POLL_RATE(100);                           // The time period to poll after sending a AF_Data_Request to poll for acknowledgement of recieved messages.
  myCC2530.ZDO_SET_REJOIN_PARAMETERS(900000, 30000);                  // Set Rejoin backoff and scan duration from MT on loss of parent.  Values are in milliseconds.
}

/********************************************************************
   Loop
 ********************************************************************
*/
void loop()
{
  /* -----------------------------------------------------------------
    Wake on PC Interrupt from CC2530
    ------------------------------------------------------------------
  */
  if (myCC2530.PCInterrupt_SRDY_Pin == true)
  {
    Serial.println("~~PC Interrupt SRDY Pin~~");
    Process_Incoming_Message(150);
    myCC2530.PCInterrupt_SRDY_Pin = false;
  }
  /* -----------------------------------------------------------------
    Wake on External Interrupt
    ------------------------------------------------------------------
  */
  if (myCC2530.EXTInterrupt_D2_Pin == true)
  {
    Serial.println("~~External Interrupt D2 Pin~~");
    if (myCC2530.DeviceState == 0x06) // State: End device
    {
      uint8_t Data = 0x08;                                              // Data to send
      myCC2530.AF_DATA_REQUEST(0x00, 0x00, Data, sizeof(Data));         // AF_DATA_REQUEST(Short Address, Short Address, Variable to send, Length).  The coordinators address is always 0x0000.
      myCC2530.AF_DATA_CONFIRM(150);                                    // Function returns true if message acknowledgement (ACK) is success and false if it fails. The maximum amout of time to wait for message acknowledgement (ACK).  See "Z-stack Duty Cycle Analysis" Application Note AN114.  Within 100 ms, as many as 3 application retries can be attempted.
    }
    else if (myCC2530.DeviceState == 0x0C) // State: Backoff
    {
      Serial.println("Device State Backoff");
    }
    myCC2530.EXTInterrupt_D2_Pin = false;
  }

  /* -----------------------------------------------------------------
    Sleep
    ------------------------------------------------------------------
  */
  myCC2530.POLL_TIMEOUT(150);
  SetPinsSleep();
  myCC2530.SLEEP_EXT_INTERRUPT();                                     // Puts the Atmega328P to sleep until an External Interrupt occurs on pin D2 or a PC Interrupt on pin SRDY
  SetPinsWake();
}

/********************************************************************
   Other Functions
 ********************************************************************
*/

/* ------------------------------------------------------------------
   Initialise CC2530
   ------------------------------------------------------------------
*/
void Init_CC2530()
{
  myCC2530.POWER_UP();                                                // Power up the CC2530 (wake on reset).

  if (digitalRead(InitButton) == LOW)
  {
    myCC2530.COMMISSION();                                            // Clears the configuration and network state then writes the new configuration parameters to the CC2530 non-volitile (NV) memory.  This should only be run once on initial setup.
  }

  myCC2530.AF_REGISTER(0x01);                                         // Register Endpoint
  myCC2530.ZDO_STARTUP_FROM_APP(60000);                               // Starts the CC2530 in the network
  delay(2000);
}

/* ------------------------------------------------------------------
   Process_Incoming_Message
   Description: Processes incoming data from the CC2530 on PC interrupt (SRDY LOW)
   ------------------------------------------------------------------
*/
void Process_Incoming_Message(unsigned long WaitTime)
{
  unsigned long TimeNow = 0;
  TimeNow = millis();
  while (millis() - TimeNow < WaitTime)
  {
    myCC2530.POLL();
    if (myCC2530.NEW_DATA())
    {
      if (myCC2530.AF_INCOMING_MSG())                                     // Function returns true if a AF_INCOMING_MSG is received
      { // Process Incoming Messages in this block of code
        myCC2530.LINK_QUALITY();                                          // Function prints the Short Address and Link Quality from a received message.  The link quality is from the last Hop to the receiving device.
        uint8_t ReceivedData;                                             // Variable to receive data
        myCC2530.COPY_PAYLOAD(ReceivedData);                              // Copies the received payload to the variable "ReceivedData"
        Serial.print("Received Data: ");
        Serial.println(ReceivedData);
        Serial.println("");
      }
    }
  }
}

/* ------------------------------------------------------------------
   PC Interrupt
   Description: Required for SLEEP Function.  Set PC interrupt flag variable when SRDY goes low.
   ------------------------------------------------------------------
*/
ISR (PCINT0_vect)
{
  myCC2530.PCInterrupt_SRDY_Pin = true;
}

/* ------------------------------------------------------------------
   SetPinsSleep
   Description: Set pin states Output LOW before Sleep
   ------------------------------------------------------------------
*/
void SetPinsSleep()
{
  for (byte i = 3; i <= 6; i++)                                           // Set pins output low
  {
    pinMode (i, OUTPUT);
    digitalWrite (i, LOW);
  }

  for (byte i = A0; i <= A5; i++)
  {
    pinMode (i, OUTPUT);
    digitalWrite (i, LOW);
  }
}

/* ------------------------------------------------------------------
   SetPinsWake
   Description: Set pins Default inputs after sleep
   ------------------------------------------------------------------
*/
void SetPinsWake()
{
  for (byte i = 3; i <= 6; i++)                                       // Set pins default input
  {
    pinMode (i, INPUT);
  }

  for (byte i = A0; i <= A5; i++)
  {
    pinMode (i, INPUT);
  }
}
