/*
    Example: VT1100 DHT11
    Description: A low powered Temperature and Humidity sensor example

    The device goes through the following loop sequence:
    1. Take sensor readings
    2. Send data
    3. Sleep
*/

/* ------------------------------------------------------------------
   Includes
   ------------------------------------------------------------------
*/
#include <VT1100_Mini.h>
#include <SPI.h>
#include <dht.h>

/* ------------------------------------------------------------------
   Class Instances
   ------------------------------------------------------------------
*/
CC2530 myCC2530;                                                      // Library VT1100 Mini Class Instance
dht DHT;                                                              // ****Instance of the dht class called DHT

/* ------------------------------------------------------------------
   Pins
   ------------------------------------------------------------------
*/
#define PIN_EN      7                                                 // Enable
#define PIN_SRDY    8                                                 // (P0_4) SRDY: Slave ready.
#define PIN_RES     9                                                 // (RST) RESET
#define PIN_SS_MRDY 10                                                // (P0_3) (P1_4) SS: Slave select.  MRDY: Master Ready.  Active Low.  Connect SS and MRDY together
#define PIN_MOSI    11                                                // (P1_6) SPI MO: Master-output slave-input data
#define PIN_MISO    12                                                // (P1_7) SPI MI: Master-input slave-output data
#define PIN_SCK     13                                                // (P1_5) SPI Serial Clock

#define Button      2                                                 // Commission button
#define DHT11_PIN   3                                                 // DHT11 input pin

/* ------------------------------------------------------------------
   Millis - Timing
   ------------------------------------------------------------------
*/
//unsigned long period = 30000;
unsigned long time_now = 0;

/* ------------------------------------------------------------------
   Global Veriables
   ------------------------------------------------------------------
*/
uint8_t seqCount = 0;                                                 // Used for keeping track of message transaction sequences
uint8_t seqNumber = 0;                                                // Used to match read attribute requests to read attribute responses

/* ------------------------------------------------------------------
   ZCL: Read Attribute Response Commands
   ------------------------------------------------------------------
*/
const uint8_t ModelIdentifier[] = {'V', 'T', '1', '1', '0', '0', 'D', 'H', 'T', '1', '1'};                               // Model Identifier: VT1100DHT11
const uint8_t ManufacturerName[] = {'V', 'e', 'r', 't', 'o', 'r', 'i', 'x'};                                             // Manufacturer Name: Vertorix
const uint8_t PowerSource = 3;                                                                                           // Power Source: 0 = Unknown, 1 = mains(single phase), 2 = mains(3 phase), 3 = battery, 4 = DC source.
const uint8_t ApplicationVersion = 1;                                                                                    // Application Version
const uint8_t ZCLVersion = 1;                                                                                            // ZCL version
const uint8_t StackVersion = 2;                                                                                          // Stack version
const uint8_t HWVersion = 1;                                                                                             // Hardware version
const uint8_t Datecode[] = {'2', '0', '2', '1', '0', '2', '0', '3'};                                                     // Date Code
const uint8_t SoftwareBuildID[] = {'1', '2', '0', '0', '-', '0', '0', '0', '1'};                                         // Software Build ID

/********************************************************************
   Setup
 ********************************************************************
*/
void setup()
{
  Serial.begin(115200);
  delay(100);

  pinMode(PIN_EN, OUTPUT);
  digitalWrite(PIN_EN, LOW);

  SPI.begin();

  // Parameters for CC2530
  myCC2530.SetPANID(0xffff);                                          // PAN ID.  Two bytes set between 0x0000 and 0x3FFF.  Examples: 0x00A1, 0x00A2 or 0x00A3.
  myCC2530.SetLOGICAL_TYPE(0x02);                                     // Device type.  Examples: Coordinator = 0x00, Router = 0x01 or End Device = 0x02
  myCC2530.SetCHANLIST(11);                                           // Wireless Channel. Examples: 11 to 26 or 0xFF All Channels
  myCC2530.SetPOLL_FAILURE_RETRIES(3);                                // Set poll failure retries

  Init_CC2530();
  myCC2530.WriteNV_POLLRATE(2000);                                    // Writes to non-volitile memory to set Poll Rate. Must be run after joining process.  The time period in milliseconds to poll the parent for messages.
  myCC2530.WriteNV_QUEUED_POLL_RATE(100);                             // The time period to poll again for more data if the parent does have data.
  myCC2530.WriteNV_RESPONSE_POLL_RATE(100);                           // The time period to poll after sending a AF_Data_Request to poll for acknowledgement of recieved messages.
  myCC2530.ZDO_SET_REJOIN_PARAMETERS(900000, 30000);                  // Set Rejoin backoff and scan duration from MT on loss of parent.  Values are in milliseconds.
}

/********************************************************************
   Loop
 ********************************************************************
*/
void loop()
{
  /* -----------------------------------------------------------------
    Wake on PC Interrupt from CC2530
    ------------------------------------------------------------------
  */
  if (myCC2530.PCInterrupt_SRDY_Pin == true)                            // CC2530 has set SRDY pin low and has a message to send to Atmega328
  {
    Serial.println("~~PC Interrupt SRDY Pin~~");
    Process_Incoming_Message(150);
    myCC2530.PCInterrupt_SRDY_Pin = false;                              // Set PC interrupt flag to false before sleeping
  }
  /* -----------------------------------------------------------------
    Wake on Watchdog Timer Interrupt
    ------------------------------------------------------------------
  */
  else
  {
    /* ------------------------------------------------------------------
      Sensor Readings
      -------------------------------------------------------------------
    */
    digitalWrite(PIN_EN, HIGH);                                        // Power on sensors

    delay(2000);
    int chk = DHT.read11(DHT11_PIN);                                   // Read DHT11 sensor
    delay(1500);                                                       // Sensor needs 1.5 sec to take readings

    digitalWrite(PIN_EN, LOW);                                         // Power off sensors

    /* ------------------------------------------------------------------
      Send Data
      -------------------------------------------------------------------
    */
    if (myCC2530.DeviceState == 0x06) // State: End device
    {
      ZCLFrame_ReportAttributes_Temp();                                // Send Temperature ZCL report attribute
      myCC2530.AF_DATA_CONFIRM(150);
      ZCLFrame_ReportAttributes_Humidity();                            // Send Humidity ZCL report attribute
      myCC2530.AF_DATA_CONFIRM(150);                                   // Function returns true if message acknowledgement (ACK) is success and false if it fails. The maximum amout of time to wait for message acknowledgement (ACK).  See "Z-stack Duty Cycle Analysis" Application Note AN114.  Within 100 ms, as many as 3 application retries can be attempted.
    }
    else if (myCC2530.DeviceState == 0x0C) // State: Backoff
    {
      Serial.println("Device State Backoff");
    }
    myCC2530.GlobalSleepCount = 8;                                     // Set Global Sleep Counter
  }
  /* -----------------------------------------------------------------
    Sleep
    ------------------------------------------------------------------
  */
  Serial.print("GlobalSleepCount: ");
  Serial.println(myCC2530.GlobalSleepCount);
  myCC2530.POLL();                                                    // Poll before sleep to ensure messages are cleared
  SetPinsSleep();
  myCC2530.SLEEP_WATCHDOG_TIMER(myCC2530.GlobalSleepCount);           // Sleep using Watchdog Timer
  SetPinsWake();
}

/********************************************************************
   Other Functions
 ********************************************************************
*/

/* ------------------------------------------------------------------
   Initialise CC2530
   ------------------------------------------------------------------
*/
void Init_CC2530()
{
  myCC2530.POWER_UP();                                                // Power up the CC2530 (wake on reset).
  SetTXPower();

  pinMode(Button, INPUT_PULLUP);
  if (digitalRead(Button) == LOW)
  {
    AF_REGISTER(0x01);                                                // Register Endpoint 1 ZCL
    myCC2530.ZDO_STARTUP_FROM_APP(12000);                             // Starts the CC2530 in the network
    myCC2530.POLL_TIMEOUT(2000);
    LEAVE_REQ();
    myCC2530.POLL_TIMEOUT(2000);

    myCC2530.COMMISSION();                                            // Clears the configuration and network state then writes the new configuration parameters to the CC2530 non-volitile (NV) memory.  This should only be run once on initial setup.
    AF_REGISTER(0x01);                                                // Register Endpoint 1 ZCL
    myCC2530.ZDO_STARTUP_FROM_APP(60000);                             // Starts the CC2530 in the network
    myCC2530.POLL_TIMEOUT(5000);
    AF_REGISTER(0x01);                                                // Register Endpoint 1 ZCL
    myCC2530.ZDO_STARTUP_FROM_APP(60000);                             // Starts the CC2530 in the network
    Interview(30000);
  }

  if (digitalRead(Button) == HIGH)
  {
    AF_REGISTER(0x01);
    myCC2530.ZDO_STARTUP_FROM_APP(60000);                             // Starts the CC2530 in the network
    myCC2530.POLL_TIMEOUT(5000);
  }
}

/* ------------------------------------------------------------------
   ZDO_MGMT_LEAVE_REQ
   ------------------------------------------------------------------
*/
void LEAVE_REQ()
{
  uint8_t IEEEAddr[8];
  myCC2530.GET_IEEE_ADDRESS(IEEEAddr);
  uint8_t DestAddr[2] = {0xFC, 0xFF}; // Broadcast to coordinator and all Routers
  myCC2530.ZDO_MGMT_LEAVE_REQ(DestAddr, IEEEAddr);
}

/* ------------------------------------------------------------------
   Set TX Power Function
   ------------------------------------------------------------------
*/
void SetTXPower()
{
  uint8_t TXPower[] = {0x01, 0x21, 0x14, 0x04};

  myCC2530.WRITE_DATA(TXPower);
  Serial.println("~~TX power set~~");
}

/* ------------------------------------------------------------------
   Interview
   ------------------------------------------------------------------
*/
void Interview(unsigned long WaitTime)
{
  time_now = millis();
  while (millis() - time_now < WaitTime)
  {
    myCC2530.POLL();                                                  // Need to constantly Poll for data from the CC2530 to the application processor

    if (myCC2530.AF_INCOMING_MSG())                                   // Function returns true if a AF_INCOMING_MSG is received
    { // Process Incoming Messages in this block of code
      myCC2530.LINK_QUALITY();                                        // Function prints the Short Address and Link Quality from a received message.  The link quality is from the last Hop to the receiving device.

      uint8_t ClusterID[2];
      ClusterID[0] = myCC2530.ReceivedBytes[5];
      ClusterID[1] = myCC2530.ReceivedBytes[6];

      if (ClusterID[0] == 0x00 && ClusterID[1] == 0x00)
      {
        ZCL_Interview();                                              // Send ZCL Read attributes response.  This is used in the interview process.
      }
      myCC2530.LINK_QUALITY();                                        // Function prints the Short Address and Link Quality from a received message.  The link quality is from the last Hop to the receiving device.
    }
  }
}

/* ------------------------------------------------------------------
   AF_Register Function
   ------------------------------------------------------------------
*/
void AF_REGISTER(uint8_t EndPoint)
{
  uint8_t Len = 0x0F;                                               // total length after Cmd1 excl. XOR
  uint8_t Cmd0 = 0x24;
  uint8_t Cmd1 = 0x00;
  uint8_t AppEndPoint = EndPoint;                                   // Register End Point
  uint8_t AppProfileID0 = 0x04;
  uint8_t AppProfileID1 = 0x01;
  uint8_t DeviceID0 = 0x00;
  uint8_t DeviceID1 = 0x00;
  uint8_t DeviceVer = 0x01;
  uint8_t LatencyReq = 0x00;
  uint8_t AppNumInClusters = 0x03;
  uint8_t InCluster0 = 0x00;
  uint8_t InCluster1 = 0x00;
  uint8_t InCluster2 = 0x02;
  uint8_t InCluster3 = 0x04;
  uint8_t InCluster4 = 0x05;
  uint8_t InCluster5 = 0x04;
  uint8_t AppNumOutClusters = 0x00;

  // Make array
  uint8_t AFRegister[] = {Len, Cmd0, Cmd1, AppEndPoint, AppProfileID0, AppProfileID1, DeviceID0, DeviceID1, DeviceVer, LatencyReq, AppNumInClusters, InCluster0, InCluster1, InCluster2, InCluster3, InCluster4, InCluster5, AppNumOutClusters};

  DEBUG_SERIAL.println(F("AF_REGISTER SREQ"));
  // SREQ
  digitalWrite(PIN_SS_MRDY, LOW);
  while (digitalRead(PIN_SRDY) == HIGH) {};
  SPI.beginTransaction(SPISettings(2000000, MSBFIRST, SPI_MODE0));
  for (int i = 0; i < sizeof(AFRegister); i++)
  {
    SPI.transfer(AFRegister[i]);
  }
  while (digitalRead(PIN_SRDY) == LOW) {};

  myCC2530.SRSP();
}

/* ------------------------------------------------------------------
   ZCL Read Attributes Response
   ------------------------------------------------------------------
*/
void ZCLFrame_ReadAttributesResponse(uint8_t Data[], uint8_t arraysize)
{
  // ZCL Header
  uint8_t FrameControl = 0x18;                                      // Bitfield that defines the command type and other relevant information in the ZCL command
  uint8_t SequenceNumber = seqNumber;                               // A sequence number used to correlate a ZCL command with a ZCL response.
  uint8_t CommandID = 0x01;                                         // Read Attributes Response
  const uint8_t ZCLHeader[] = {FrameControl, SequenceNumber, CommandID};  // ZCL Header

  // ZCL Payload
  uint8_t ZCLPayload[30];
  memcpy(ZCLPayload, Data, arraysize);

  // ZCL Frame
  uint8_t ZCLFrame[sizeof(ZCLHeader) + sizeof(ZCLPayload)];
  memcpy(ZCLFrame, ZCLHeader, sizeof(ZCLHeader));
  memcpy(ZCLFrame + sizeof(ZCLHeader), ZCLPayload, sizeof(ZCLPayload));
  uint8_t ZCLFrameLength = sizeof(ZCLHeader) + arraysize;

  // Send ZCL Frame
  myCC2530.SetAF_DATA_REQUEST(0x01, 0x01, 0x00, 0x00, 0x00, 0x10, 0x30);
  myCC2530.AF_DATA_REQUEST(0x00, 0x00, ZCLFrame, ZCLFrameLength);
}

/* ------------------------------------------------------------------
   ZCL_Interview
   ------------------------------------------------------------------
*/
void ZCL_Interview()
{
  uint8_t ControlField = myCC2530.ReceivedBytes[20];
  seqNumber = myCC2530.ReceivedBytes[21];
  uint8_t Command = myCC2530.ReceivedBytes[22];
  uint8_t Attribute[2];
  Attribute[0] = myCC2530.ReceivedBytes[23];
  Attribute[1] = myCC2530.ReceivedBytes[24];

  if (Attribute[0] == 0x05 && Attribute[1] == 0x00)
  {
    uint8_t Status = 0x00;
    uint8_t DataType = 0x42; // Char String
    uint8_t Length = sizeof(ModelIdentifier);

    uint8_t Payload[5 + Length] = {Attribute[0], Attribute[1], Status, DataType, Length};
    memcpy(Payload + 5, ModelIdentifier, sizeof(ModelIdentifier));

    ZCLFrame_ReadAttributesResponse(Payload, sizeof(Payload));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x04 && Attribute[1] == 0x00)
  {
    uint8_t Status = 0x00;
    uint8_t DataType = 0x42; // Char String
    uint8_t Length = sizeof(ManufacturerName);

    uint8_t Payload[5 + Length] = {Attribute[0], Attribute[1], Status, DataType, Length};
    memcpy(Payload + 5, ManufacturerName, sizeof(ManufacturerName));

    ZCLFrame_ReadAttributesResponse(Payload, sizeof(Payload));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x07 && Attribute[1] == 0x00)
  {
    uint8_t _PowerSource[] = {0x07, 0x00, 0x00, 0x30, PowerSource};
    ZCLFrame_ReadAttributesResponse(_PowerSource, sizeof(_PowerSource));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x01 && Attribute[1] == 0x00)
  {
    uint8_t _ApplicationVersion[] = {0x01, 0x00, 0x00, 0x20, ApplicationVersion};
    ZCLFrame_ReadAttributesResponse(_ApplicationVersion, sizeof(_ApplicationVersion));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x00 && Attribute[1] == 0x00)
  {
    uint8_t _ZCLVersion[] = {0x00, 0x00, 0x00, 0x20, ZCLVersion};
    ZCLFrame_ReadAttributesResponse(_ZCLVersion, sizeof(_ZCLVersion));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x02 && Attribute[1] == 0x00)
  {
    uint8_t _StackVersion[] = {0x02, 0x00, 0x00, 0x20, StackVersion};
    ZCLFrame_ReadAttributesResponse(_StackVersion, sizeof(_StackVersion));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x03 && Attribute[1] == 0x00)
  {
    uint8_t _HWVersion[] = {0x03, 0x00, 0x00, 0x20, HWVersion};
    ZCLFrame_ReadAttributesResponse(_HWVersion, sizeof(_HWVersion));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x06 && Attribute[1] == 0x00)
  {
    uint8_t Status = 0x00;
    uint8_t DataType = 0x42; // Char String
    uint8_t Length = sizeof(Datecode);

    uint8_t Payload[5 + Length] = {Attribute[0], Attribute[1], Status, DataType, Length};
    memcpy(Payload + 5, Datecode, sizeof(Datecode));

    ZCLFrame_ReadAttributesResponse(Payload, sizeof(Payload));
    myCC2530.RECV_CALLBACK();
  }
  else if (Attribute[0] == 0x00 && Attribute[1] == 0x40)
  {
    uint8_t Status = 0x00;
    uint8_t DataType = 0x42; // Char String
    uint8_t Length = sizeof(SoftwareBuildID);

    uint8_t Payload[5 + Length] = {Attribute[0], Attribute[1], Status, DataType, Length};
    memcpy(Payload + 5, SoftwareBuildID, sizeof(SoftwareBuildID));

    ZCLFrame_ReadAttributesResponse(Payload, sizeof(Payload));
    myCC2530.RECV_CALLBACK();
  }
}

/* ------------------------------------------------------------------
   ZCL Report Temperature
   ------------------------------------------------------------------
*/

void ZCLFrame_ReportAttributes_Temp()
{
  // ZCL Header
  uint8_t FrameControl = 0x18;                                      // Bitfield that defines the command type and other relevant information in the ZCL command
  uint8_t SequenceNumber = seqCount;                                // A sequence number used to correlate a ZCL command with a ZCL response.
  seqCount++;
  uint8_t CommandID = 0x0a;                                         // CommandID: Report Attributes
  uint8_t ZCLHeader[] = {FrameControl, SequenceNumber, CommandID};  // ZCL Header

  // ZCL Payload
  uint8_t AttributeID_MeasuredVal[2] = {0x00, 0x00};                // Attribute ID: Measured Value
  uint8_t DataType = 0x29;                                          // Data Type: 16 bit integer Temperature measurement
  int Temp = DHT.temperature * 100;                                 // Temperature value
  uint8_t ZCLPayload[] = {AttributeID_MeasuredVal[0], AttributeID_MeasuredVal[1], DataType, lowByte(Temp), highByte(Temp)}; // ZCL Payload

  // ZCL Frame
  uint8_t ZCLFrame[sizeof(ZCLHeader) + sizeof(ZCLPayload)];
  memcpy(ZCLFrame, ZCLHeader, sizeof(ZCLHeader));
  memcpy(ZCLFrame + sizeof(ZCLHeader), ZCLPayload, sizeof(ZCLPayload));

  // Send ZCL Frame
  myCC2530.SetAF_DATA_REQUEST(0x01, 0x01, 0x02, 0x04, 0x00, 0x00, 0x30);  // Destination EP, Source EP, ClusterID, ClusterID, Trans ID, Options, Radius
  myCC2530.AF_DATA_REQUEST(0x00, 0x00, ZCLFrame, sizeof(ZCLFrame));       // NwkAddr, NwkAddr, Payload to Send, Length of Payload
}

/* ------------------------------------------------------------------
   ZCL Report Humidity
   ------------------------------------------------------------------
*/

void ZCLFrame_ReportAttributes_Humidity()
{
  // ZCL Header
  uint8_t FrameControl = 0x18;                                      // Bitfield that defines the command type and other relevant information in the ZCL command
  uint8_t SequenceNumber = seqCount;                                // A sequence number used to correlate a ZCL command with a ZCL response.
  seqCount++;
  uint8_t CommandID = 0x0a;                                         // CommandID: Report Attributes
  uint8_t ZCLHeader[] = {FrameControl, SequenceNumber, CommandID};  // ZCL Header

  // ZCL Payload
  uint8_t AttributeID_MeasuredVal[2] = {0x00, 0x00};                // Attribute ID: Measured Value
  uint8_t DataType = 0x21;                                          // Data Type: 16 bit integer Humidity Measurement
  int Humidity = DHT.humidity * 100;                                // Humidity value
  uint8_t ZCLPayload[] = {AttributeID_MeasuredVal[0], AttributeID_MeasuredVal[1], DataType, lowByte(Humidity), highByte(Humidity)}; // ZCL Payload

  // ZCL Frame
  uint8_t ZCLFrame[sizeof(ZCLHeader) + sizeof(ZCLPayload)];
  memcpy(ZCLFrame, ZCLHeader, sizeof(ZCLHeader));
  memcpy(ZCLFrame + sizeof(ZCLHeader), ZCLPayload, sizeof(ZCLPayload));

  // Send ZCL Frame
  myCC2530.SetAF_DATA_REQUEST(0x01, 0x01, 0x05, 0x04, 0x00, 0x00, 0x30);  // Destination EP, Source EP, ClusterID, ClusterID, Trans ID, Options, Radius
  myCC2530.AF_DATA_REQUEST(0x00, 0x00, ZCLFrame, sizeof(ZCLFrame));       // NwkAddr, NwkAddr, Payload to Send, Length of Payload
}

/* ------------------------------------------------------------------
   Process_Incoming_Message
   Description: Processes incoming data from the CC2530 on PC interrupt (SRDY LOW)
   ------------------------------------------------------------------
*/
void Process_Incoming_Message(unsigned long WaitTime)
{
  unsigned long TimeNow = 0;
  TimeNow = millis();
  while (millis() - TimeNow < WaitTime)
  {
    myCC2530.POLL();
    if (myCC2530.NEW_DATA())
    {
      if (myCC2530.AF_INCOMING_MSG())                                     // Function returns true if a AF_INCOMING_MSG is received
      { // Process Incoming Messages in this block of code
        myCC2530.LINK_QUALITY();                                          // Function prints the Short Address and Link Quality from a received message.  The link quality is from the last Hop to the receiving device.
        uint8_t ReceivedData;                                             // Variable to receive data
        myCC2530.COPY_PAYLOAD(ReceivedData);                              // Copies the received payload to the variable "ReceivedData"
        Serial.print("Received Data: ");
        Serial.println(ReceivedData);
        Serial.println("");
      }
    }
  }
}

/* ------------------------------------------------------------------
   PC Interrupt
   Description: Required for SLEEP Function.  Set PC interrupt flag variable when SRDY goes low.
   ------------------------------------------------------------------
*/
ISR (PCINT0_vect)
{
  myCC2530.PCInterrupt_SRDY_Pin = true;
}

/* ------------------------------------------------------------------
   SetPinsSleep
   Description: Set pin states Output LOW before Sleep
   ------------------------------------------------------------------
*/
void SetPinsSleep()
{
  for (byte i = 2; i <= 6; i++)                                           // Set pins output low
  {
    pinMode (i, OUTPUT);
    digitalWrite (i, LOW);
  }

  for (byte i = A0; i <= A5; i++)
  {
    pinMode (i, OUTPUT);
    digitalWrite (i, LOW);
  }
}

/* ------------------------------------------------------------------
   SetPinsWake
   Description: Set pins Default inputs after sleep
   ------------------------------------------------------------------
*/
void SetPinsWake()
{
  for (byte i = 2; i <= 6; i++)                                       // Set pins default input
  {
    pinMode (i, INPUT);
  }

  for (byte i = A0; i <= A5; i++)
  {
    pinMode (i, INPUT);
  }
}
